﻿package
{
	import flash.display.MovieClip;
	import flash.events.Event;
	import flash.events.EventDispatcher;
	import flash.events.IOErrorEvent;
	import flash.events.MouseEvent;
	import flash.events.ProgressEvent;
	import flash.events.TimerEvent;
	import flash.media.Sound;
	import flash.media.SoundChannel;
	import flash.media.ID3Info;
	import flash.net.URLRequest;
	import flash.utils.Timer;
	import flash.text.TextField;
	import flash.text.TextFieldAutoSize;
	import caurina.transitions.Tweener;
	
	public class SimpleMP3Player extends MovieClip
	{
		// Create some variables that we need
		private var mp3File:Sound;
		private var mp3FilePosition:SoundChannel;
		private var id3Data:ID3Info;
		private var id3InfoAvailable = false;
		
		private var updateTime:Timer;
		
		private var playingSound:Boolean = false;
		private var soundPosition:Number;
		private var songReachedEnd:Boolean = false;
		
		/*
		*	buttonPos variable stores allButtons.bs y cordinates
		*
		*	buttonPos[0] = playUp
		*	buttonPos[1] = playOver
		*	buttonPos[2] = playDown
		*	buttonPos[3] = playDis
		*	buttonPos[4] = pauseUp
		*	buttonPos[5] = pauseOver
		*	buttonPos[6] = pauseDown
		*	buttonPos[7] = pauseDis
		*/
		private var buttonPos:Array = new Array('0', '-36', '-72', '-108', '-144', '-180', '-216', '-252');
		
		// Method executes when class is loaded
		public function SimpleMP3Player()
		{	
			// Move playerTexts movieclip out of sight
			display.playerTexts.x = -73;
			// Apply mask to display
			display.mask = displayMask;
			
			// Default buttonMode and enable is false
			bs.buttons.buttonMode = bs.buttons.enabled = false;
			// Show disabled play button when .swf executes
			bs.buttons.y = buttonPos[3];
			
			// Call our 'main' method
			loadMP3();
		}
		
		private function loadMP3():void
		{
			// Create new Sound object
			mp3File = new Sound();
			mp3File.addEventListener(ProgressEvent.PROGRESS, mp3FileLoading);
			mp3File.addEventListener(Event.COMPLETE, mp3FileLoaded);
			mp3File.addEventListener(IOErrorEvent.IO_ERROR, errorLoadingSound);
			
			// Invoked when MP3 ID3 info is available
			mp3File.addEventListener(Event.ID3, getID3Data);
			// Our mp3 that we want to laod
			mp3File.load(new URLRequest('tarzan.mp3'));
			
			// Tween preloader to display
			Tweener.addTween(display.preloader, {x: 96, time:1});
		}
		
		private function mp3FileLoading(e:ProgressEvent):void
		{
			// Calculate percent...
			var currentPercent:Number = Math.round(e.bytesLoaded / e.bytesTotal * 100);
			// ...and update percent textfield everytime ProgressEvent fires up
			display.preloader.text = 'LOADING...' + currentPercent + '%';
			
			// Test if song have loaded at least 50 percent and
			// id3InfoAvailable have to be also true
			if (currentPercent > 50 && id3InfoAvailable)
			{
				Tweener.addTween(display.preloader, {x: 200, time:1, onComplete:playMP3, onCompleteParams:[false, 0]});
				mp3File.removeEventListener(ProgressEvent.PROGRESS, mp3FileLoading);
			}
		}
		
		private function mp3FileLoaded(e:Event):void
		{
			// Remove event listener. It's no longer needed
			mp3File.removeEventListener(Event.COMPLETE, mp3FileLoaded);
			mp3File.removeEventListener(ProgressEvent.PROGRESS, updateBufferField);
			// Slide buffer percent textfield out of sight
			Tweener.addTween(display.playerTexts.buffer, {x: 200, time:1});
		}
		
		private function errorLoadingSound(e:IOErrorEvent):void
		{
			trace('Error loading sound: ' + e);
		}
		
		private function getID3Data(e:Event):void
		{
			// This variable have to be true or song won't start to play
			id3InfoAvailable = true;
			// Add ID3 data to id3Data variable
			id3Data = mp3File.id3;
		}
		
		private function playMP3(useSp:Boolean, sp:Number):void
		{			
			if (useSp) mp3FilePosition = mp3File.play(sp);
			else mp3FilePosition = mp3File.play();
			
			playingSound = true;
			mp3FilePosition.addEventListener(Event.SOUND_COMPLETE, songFinished);
			mp3File.addEventListener(ProgressEvent.PROGRESS, updateBufferField);
			
			// Enable buttons and set button image to playUp
			bs.buttons.buttonMode = bs.buttons.enabled = true;
			bs.buttons.y = buttonPos[0];
			
			// Add necessary button sprite EventListeners
			bs.buttons.addEventListener(MouseEvent.MOUSE_OVER, mouseOverBs);
			bs.buttons.addEventListener(MouseEvent.MOUSE_OUT, mouseOutBs);
			bs.buttons.addEventListener(MouseEvent.CLICK, mouseClickBs);
												
			// Create new Timer with 100 millisecond interval
			updateTime = new Timer(100);

			// Call getMP3Time method every 100 millisecond
			updateTime.addEventListener(TimerEvent.TIMER, getMP3Time);
			// Start Timer
			updateTime.start();
			
			// Create string si(song information) to store artist name and song name
			var si:String = id3Data.artist + ' - ' + id3Data.songName;
			// Convert string to uppercase
			si = si.toUpperCase();
			display.playerTexts.songInfo.autoSize = TextFieldAutoSize.LEFT;
			display.playerTexts.songInfo.wordWrap = false;
			display.playerTexts.songInfo.text = si;
			
			Tweener.addTween(display.playerTexts, {x: 60, time:1});
		}
		
		private function updateBufferField(e:ProgressEvent):void
		{
			var currentPercent:Number = Math.round(e.bytesLoaded / e.bytesTotal * 100);
			display.playerTexts.buffer.text = currentPercent + '%';
		}
		
		private function getMP3Time(e:TimerEvent):void
		{
			// Basic calculation from milliseconds to minutes and seconds
			var totalMinutes = Math.floor(mp3File.length / 1000 / 60);
  			var totalSeconds = Math.floor(mp3File.length / 1000) % 60;
			var currentMinutes = Math.floor(mp3FilePosition.position / 1000 / 60);
  			var currentSeconds = Math.floor(mp3FilePosition.position / 1000) % 60;
			
			// If total- or currentseconds are less than 10 seconds add extra zero :)
			if (totalSeconds < 10) totalSeconds = '0' + totalSeconds;
			if (currentSeconds < 10) currentSeconds = '0' + currentSeconds;
			
			// Update our times textfield in playerTexts with new values
			display.playerTexts.times.text = currentMinutes + ':' + currentSeconds + '/' + totalMinutes + ':' + totalSeconds;
		}
		
		private function songFinished(e:Event):void
		{
			// Remove SOUND_COMPLETE and TIMER event listeners
			// Is user want to play again playMP3 method creates same event listeners again
			mp3FilePosition.removeEventListener(Event.SOUND_COMPLETE, songFinished);
			updateTime.removeEventListener(TimerEvent.TIMER, getMP3Time);
			
			bs.buttons.y = buttonPos[3];
			mp3FilePosition.stop();
			
			// Set playingSound to false. Just in case
			playingSound = false;
			songReachedEnd = true;
			
			// Set soundPosition to zero
			soundPosition = 0;
			// Write to times textfield
			display.playerTexts.times.text = 'PLAY AGAIN?';
		}
		
		/*
		*	allButtons.bs event methods begin
		*/
		
		// mouseOverBs event fires up everythime mouse is moved over the button
		private function mouseOverBs(e:MouseEvent):void
		{
			// If playingSound is true set button state to pauseOver(buttonPos[5])
			if (playingSound) bs.buttons.y = buttonPos[5];
			// Else button state is playOver(buttonPos[1])
			else bs.buttons.y = buttonPos[1];
			
			// But if song has finished playing show button playOver(buttonPos[1])
			if (songReachedEnd && !playingSound) bs.buttons.y = buttonPos[1];
		}
		
		// mouseOutBs event fires up everythime mouse is moved out from the button
		private function mouseOutBs(e:MouseEvent):void
		{	
			// If playingSound is true set button state to playUp(buttonPos[0])		
			if (playingSound) bs.buttons.y = buttonPos[0];
			// If playingSound is true set button state to pauseUp(buttonPos[4])
			else bs.buttons.y = buttonPos[4];
			
			// But if song has finished playing show button playDis(buttonPos[3])
			if (songReachedEnd && !playingSound) bs.buttons.y = buttonPos[3];
		}
		
		// mouseClickBs event fires up everytime mouse is clicked over the button
		private function mouseClickBs(e:MouseEvent):void
		{
			// If playingSound is true...
			if (playingSound)
			{
				//...set button state to pauseUp
				bs.buttons.y = buttonPos[4];
				// ...store current sound position from SoundChannel object to soundPosition
				soundPosition = mp3FilePosition.position;
				// ...stop our Timer
				updateTime.stop();
				// ...stop our song
				mp3FilePosition.stop();
				// ...set playingSound to false
				playingSound = false;
				// ...set times textfield to PAUSE
				display.playerTexts.times.text = 'PAUSE';
			}
			// If playingSound is set to false...
			else if (!playingSound)
			{
				// ...set button state to playDown(buttonPos[2])
				bs.buttons.y = buttonPos[2];
				// ...call our playMP3 method
				playMP3(true, soundPosition);
				// ...set playingSound to true
				playingSound = true;
			
			}
			// If song has ended and user want to play again...
			else if (songReachedEnd)
			{
				// ...set songReachedEnd to false
				songReachedEnd = false;
				// ...call playMP3 method and send info that we don't want
				// to use soundPosition. We wan't to start song from the begin
				playMP3(false, 0);
			}
		}
	}
}